@@ -39,12 +39,14 @@
 #include <libgames-support/games-files.h>
 #include <libgames-support/games-frame.h>
 #include <libgames-support/games-gridframe.h>
+#include <libgames-support/games-gtk-compat.h>
 #include <libgames-support/games-help.h>
 #include <libgames-support/games-preimage.h>
 #include <libgames-support/games-runtime.h>
 #include <libgames-support/games-scores.h>
 #include <libgames-support/games-scores-dialog.h>
 #include <libgames-support/games-stock.h>
+#include <libgames-support/games-fullscreen-action.h>
 
 #ifdef WITH_SMCLIENT
 #include <libgames-support/eggsmclient.h>
@@ -107,7 +109,6 @@ static GtkWidget *preview_widgets[MAXNPIECES];
 static GtkWidget *menubar;
 static GtkWidget *scoreitem;
 static GtkAction *fullscreen_action;
-static GtkAction *leavefullscreen_action;
 
 /* These keep track of what we put in the main table so we
  * can reshuffle them when we change the field size. */
@@ -261,7 +262,7 @@ relay_table (void)
 static void
 refresh_pixmaps (void)
 {
-  GdkGC *gc;
+  cairo_t *cr, *cr_blank;
   GdkPixbuf *ball_pixbuf = NULL;
 
   /* Since we get called both by configure and after loading an image.
@@ -297,20 +298,24 @@ refresh_pixmaps (void)
   g_free (warning_message);
   warning_message = NULL;
 
+  cr = gdk_cairo_create (ball_pixmap);
+  gdk_cairo_set_source_color (cr, &backgnd.color);
 
-  gc = gdk_gc_new (ball_pixmap);
-  gdk_gc_set_foreground (gc, &backgnd.color);
+  cairo_rectangle (cr, 0, 0, boxsize * 4, boxsize * 7);
+  cairo_fill (cr);
 
-  gdk_draw_rectangle (ball_pixmap, gc, TRUE, 0, 0, 4 * boxsize, 7 * boxsize);
+  gdk_cairo_set_source_pixbuf (cr, ball_pixbuf, 0, 0);
+  cairo_mask (cr, cairo_get_source (cr));
 
-  gdk_draw_pixbuf (ball_pixmap, gc,
-                   ball_pixbuf, 0, 0, 0, 0, boxsize * 4, boxsize * 7,
-                   GDK_RGB_DITHER_NORMAL, 0, 0);
+  cr_blank = gdk_cairo_create (blank_pixmap);
+  gdk_cairo_set_source_color (cr_blank, &backgnd.color);
 
-  gdk_draw_rectangle (blank_pixmap, gc, TRUE, 0, 0, boxsize, boxsize);
+  cairo_rectangle (cr_blank, 0, 0, boxsize, boxsize);
+  cairo_fill (cr_blank);
 
   g_object_unref (ball_pixbuf);
-  g_object_unref (gc);
+  cairo_destroy (cr);
+  cairo_destroy (cr_blank);
 }
 
 static void
@@ -319,12 +324,19 @@ refresh_preview_pixmaps (void)
   guint i;
   GdkPixbuf *scaled = NULL;
   GtkWidget *widget = preview_widgets[0];
+  cairo_t *cr;
+  GdkRectangle preview_rect;
 
   /* Like the refresh_pixmaps() function, we may be called before
    * the window is ready. */
   if (preview_height == 0)
     return;
 
+  preview_rect.x = 0;
+  preview_rect.y = 0;
+  preview_rect.width = preview_width;
+  preview_rect.height = preview_height;
+
   /* We create pixmaps for each of the ball colours and then
    * set them as the background for each widget in the preview array.
    * This code assumes that each preview window is identical. */
@@ -343,30 +355,32 @@ refresh_preview_pixmaps (void)
     gdk_pixbuf_fill (scaled, 0x00000000);
   }
 
-
   for (i = 0; i < 7; i++) {
     preview_pixmaps[i] = gdk_pixmap_new (gtk_widget_get_window (widget),
                                          preview_width, preview_height, -1);
-    gdk_draw_rectangle (preview_pixmaps[i],
-                        gtk_widget_get_style (widget)->bg_gc[GTK_STATE_NORMAL],
-                        TRUE, 0, 0, preview_width, preview_height);
+    cr = gdk_cairo_create (preview_pixmaps[i]);
+    gdk_cairo_set_source_color (cr, &gtk_widget_get_style (widget)->bg[GTK_STATE_NORMAL]);
+    gdk_cairo_rectangle (cr, &preview_rect);
+    cairo_fill (cr);
 
-    gdk_draw_pixbuf (preview_pixmaps[i], gtk_widget_get_style (widget)->white_gc,
-                     scaled, 0, i * preview_height, 0, 0,
-                     preview_width, preview_height,
-                     GDK_RGB_DITHER_NORMAL, 0, 0);
+    gdk_cairo_set_source_pixbuf (cr, scaled, 0, -1.0 * preview_height * i);
+    cairo_mask (cr, cairo_get_source (cr));
+
+    cairo_destroy (cr);
   }
 
   if (blank_preview_pixmap)
     g_object_unref (blank_preview_pixmap);
+
   blank_preview_pixmap = gdk_pixmap_new (gtk_widget_get_window (widget), 
                                          preview_width, preview_height, -1);
-  gdk_draw_rectangle (blank_preview_pixmap,
-                      gtk_widget_get_style (widget)->bg_gc[GTK_STATE_NORMAL], 
-                      TRUE, 0, 0, preview_width, preview_height);
+  cr = gdk_cairo_create (blank_preview_pixmap);
+  gdk_cairo_set_source_color (cr, &gtk_widget_get_style (widget)->bg[GTK_STATE_NORMAL]);
+  gdk_cairo_rectangle (cr, &preview_rect);
+  cairo_fill (cr);
 
+  cairo_destroy (cr);
   g_object_unref (scaled);
-
 }
 
 static void
@@ -816,41 +830,45 @@ key_press_event (GtkWidget * widget, GdkEventKey * event, void *d)
 static void
 draw_grid (void)
 {
-  static GdkGC *grid_gc;
+  cairo_t *cr;
+  GdkColor color;
   guint w, h;
   guint i;
+  GtkAllocation allocation;
 
-  w = draw_area->allocation.width;
-  h = draw_area->allocation.height;
+  gtk_widget_get_allocation (draw_area, &allocation);
+  w = allocation.width;
+  h = allocation.height;
 
-  if (!grid_gc) {
-    GdkColormap *cmap;
-    GdkColor color;
-
-    grid_gc = gdk_gc_new (gtk_widget_get_window (draw_area));
-
-    gdk_color_parse ("#525F6C", &color);
-    cmap = gtk_widget_get_colormap (draw_area);
-    gdk_colormap_alloc_color (cmap, &color, FALSE, TRUE);
-    gdk_gc_set_foreground (grid_gc, &color);
-  }
+  cr = gdk_cairo_create (gtk_widget_get_window (draw_area));
+  gdk_color_parse ("#525F6C", &color);
+  gdk_cairo_set_source_color (cr, &color);
+  cairo_set_line_width (cr, 1.0);
 
   for (i = boxsize; i < w; i = i + boxsize)
-    gdk_draw_line (gtk_widget_get_window (draw_area), grid_gc, i, 0, i, h);
+  {
+    cairo_move_to (cr, i + 0.5, 0 + 0.5);
+    cairo_line_to (cr, i + 0.5, h + 0.5);
+  }
 
   for (i = boxsize; i < h; i = i + boxsize)
-    gdk_draw_line (gtk_widget_get_window (draw_area), grid_gc, 0, i, w, i);
+  {
+    cairo_move_to (cr, 0 + 0.5, i + 0.5);
+    cairo_line_to (cr, w + 0.5, i + 0.5);
+  }
 
-  gdk_draw_rectangle (gtk_widget_get_window (draw_area), grid_gc, FALSE, 0, 0, 
-                      w - 1, h - 1);
+  cairo_rectangle (cr, 0.5, 0.5, w - 0.5, h - 0.5);
+  cairo_stroke (cr);
+
+  cairo_destroy (cr);
 }
 
 /* Redraw a part of the field */
-static gint
+static gboolean
 field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
 {
   GdkWindow *window = gtk_widget_get_window (widget);
-  GdkGC *gc;
+  cairo_t *cr;
   guint x_start, x_end, y_start, y_end, i, j, idx;
 
   x_start = event->area.x / boxsize;
@@ -859,7 +877,7 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
   y_start = event->area.y / boxsize;
   y_end = (event->area.y + event->area.height - 1) / boxsize + 1;
 
-  gc = gdk_gc_new (gtk_widget_get_window (draw_area));
+  cr = gdk_cairo_create (window);
 
   for (i = y_start; i < y_end; i++) {
     for (j = x_start; j < x_end; j++) {
@@ -867,26 +885,27 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
 
       idx = j + i * hfieldsize;
 
+      cairo_rectangle (cr, j * boxsize, i * boxsize, boxsize, boxsize);
+
       if (field[idx].color != 0) {
         phase = field[idx].phase;
         color = field[idx].color - 1;
 
         phase = ABS (ABS (3 - phase) - 3);
 
-        gdk_draw_drawable (window, gc, ball_pixmap,
-                           phase * boxsize,
-                           color * boxsize,
-                           j * boxsize, i * boxsize, boxsize, boxsize);
+        gdk_cairo_set_source_pixmap (cr, ball_pixmap,
+                                     (1.0 * j - phase) * boxsize,
+                                     (1.0 * i - color) * boxsize);
       } else {
-        gdk_draw_drawable (window, gc, blank_pixmap,
-                           0, 0, j * boxsize, i * boxsize, boxsize, boxsize);
+        gdk_cairo_set_source_pixmap (cr, blank_pixmap, 0, 0);
       }
+
+      cairo_fill (cr);
     }
   }
 
   if (show_cursor && cursor_x >= x_start && cursor_x <= x_end
       && cursor_y >= y_start && cursor_y <= y_end) {
-    GdkColormap *cmap;
     GdkColor color;
 
     if (((backgnd.color.red + backgnd.color.green + backgnd.color.blue) / 3) >
@@ -895,16 +914,15 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
     else
       gdk_color_parse ("#FFFFFF", &color);
 
-    cmap = gtk_widget_get_colormap (widget);
-    gdk_colormap_alloc_color (cmap, &color, FALSE, TRUE);
-    gdk_gc_set_foreground (gc, &color);
-
-    gdk_draw_rectangle (window, gc, FALSE,
-                        cursor_x * boxsize + 1, cursor_y * boxsize + 1,
-                        boxsize - 2, boxsize - 2);
+    gdk_cairo_set_source_color (cr, &color);
+    cairo_set_line_width (cr, 1.0);
+    cairo_rectangle (cr,
+                     cursor_x * boxsize + 1.5, cursor_y * boxsize + 1.5,
+                     boxsize - 2.5, boxsize - 2.5);
+    cairo_stroke (cr);
   }
 
-  g_object_unref (gc);
+  cairo_destroy (cr);
 
   draw_grid ();
 
@@ -1396,35 +1414,6 @@ set_fast_moves_callback (GtkWidget * widget, gpointer * data)
   games_conf_set_integer (KEY_PREFERENCES_GROUP, KEY_MOVE_TIMEOUT, timeout);
 }
 
-static void
-set_fullscreen_actions (gboolean fullscreen)
-{
-  gtk_action_set_sensitive (leavefullscreen_action, fullscreen);
-  gtk_action_set_visible (leavefullscreen_action, fullscreen);
-
-  gtk_action_set_sensitive (fullscreen_action, !fullscreen);
-  gtk_action_set_visible (fullscreen_action, !fullscreen);
-}
-
-static void
-fullscreen_callback (GtkAction * action)
-{
-  if (action == fullscreen_action)
-    gtk_window_fullscreen (GTK_WINDOW (app));
-  else
-    gtk_window_unfullscreen (GTK_WINDOW (app));
-}
-
-static gboolean
-window_state_callback (GtkWidget * widget, GdkEventWindowState * event)
-{
-  if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) {
-    set_fullscreen_actions ((event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) != 0);
-  }
-
-  return FALSE;
-}
-
 static void
 pref_dialog_response (GtkDialog * dialog, gint response, gpointer data)
 {
@@ -1463,7 +1452,7 @@ game_props_callback (void)
     gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (pref_dialog))),
                         vbox, FALSE, FALSE, 0);
 
-    frame = games_frame_new (_("Themes"));
+    frame = games_frame_new (_("Appearance"));
     table = gtk_table_new (2, 2, FALSE);
     gtk_container_set_border_width (GTK_CONTAINER (table), 0);
     gtk_table_set_row_spacings (GTK_TABLE (table), 6);
@@ -1664,8 +1653,6 @@ restart (void)
    * and without some sort of restricted-access storage the
    * user will always be able to change it. */
   score = games_conf_get_integer (KEY_SAVED_GROUP, KEY_SAVED_SCORE, NULL);
-  if (score < 0)
-    score = 0;
 
   buf = games_conf_get_string (KEY_SAVED_GROUP, KEY_SAVED_FIELD, NULL);
   if (buf) {
@@ -1701,10 +1688,6 @@ static const GtkActionEntry actions[] = {
    G_CALLBACK (game_help_callback)},
   {"About", GTK_STOCK_ABOUT, NULL, NULL, NULL,
    G_CALLBACK (game_about_callback)},
-  {"Fullscreen", GAMES_STOCK_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_callback)},
-  {"LeaveFullscreen", GAMES_STOCK_LEAVE_FULLSCREEN, NULL, NULL, NULL,
-   G_CALLBACK (fullscreen_callback)}
 };
 
 const char ui_description[] =
@@ -1719,7 +1702,6 @@ const char ui_description[] =
   "    </menu>"
   "    <menu action='SettingsMenu'>"
   "      <menuitem action='Fullscreen'/>"
-  "      <menuitem action='LeaveFullscreen'/>"
   "      <menuitem action='Preferences'/>"
   "    </menu>"
   "    <menu action='HelpMenu'>"
@@ -1737,11 +1719,8 @@ create_menus (GtkUIManager * ui_manager)
   gtk_action_group_add_actions (action_group, actions,
                                 G_N_ELEMENTS (actions), NULL);
 
-  fullscreen_action =
-    gtk_action_group_get_action (action_group, "Fullscreen");
-  leavefullscreen_action =
-    gtk_action_group_get_action (action_group, "LeaveFullscreen");
-  set_fullscreen_actions (FALSE);
+  fullscreen_action = GTK_ACTION (games_fullscreen_action_new ("Fullscreen", GTK_WINDOW (app)));
+  gtk_action_group_add_action_with_accel (action_group, fullscreen_action, NULL);
 
   gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
   gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, NULL);
@@ -1847,8 +1826,6 @@ main (int argc, char *argv[])
   gtk_window_set_default_size (GTK_WINDOW (app), DEFAULT_WIDTH, DEFAULT_HEIGHT);
   g_signal_connect (app, "delete-event",
                     G_CALLBACK (game_quit_callback), NULL);
-  g_signal_connect (app, "window-state-event",
-                    G_CALLBACK (window_state_callback), NULL);
   games_conf_add_window (GTK_WINDOW (app), NULL);
 
   statusbar = gtk_statusbar_new ();
@@ -1928,7 +1905,7 @@ main (int argc, char *argv[])
                          gtk_widget_get_events (draw_area) |
                          GDK_BUTTON_PRESS_MASK | GDK_KEY_PRESS_MASK);
 
-  GTK_WIDGET_SET_FLAGS (draw_area, GTK_CAN_FOCUS);
+  gtk_widget_set_can_focus (draw_area, TRUE);
   gtk_widget_grab_focus (draw_area);
 
   load_properties ();
