@@ -22,6 +22,14 @@ G_GNUC_INTERNAL void games_marshal_BOOLEAN__ENUM_INT (GClosure     *closure,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);
 
+/* BOOLEAN:STRING,UINT,ENUM (games-marshal.list:3) */
+G_GNUC_INTERNAL void games_marshal_BOOLEAN__STRING_UINT_ENUM (GClosure     *closure,
+                                                              GValue       *return_value,
+                                                              guint         n_param_values,
+                                                              const GValue *param_values,
+                                                              gpointer      invocation_hint,
+                                                              gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __games_marshal_MARSHAL_H__ */
